#!/bin/sh

# clean previous build files
rm -f *.o
rm -f bootloader.bin
rm -f Slidoor/boot/bootloader.bin
rm -f Slidoor.iso

if [ "clean" = "$1" ]; then
  exit
fi;

# compile
nasm -f elf32 bootloader.asm -o bootloader.o
gcc -m32 -fno-stack-protector -fno-pic -nostdlib -I. -c kernel.c -o kc.o

nasm -f elf include/VM86/isrs.asm -o isrs.o
nasm -f elf include/VM86/cpu.asm -o cpu.o
nasm -f elf include/VM86/vm86asm.asm -o vm86asm.o
gcc -m32 -fno-stack-protector -fno-pic -I. -c include/VM86/ints.c -o ints.o
gcc -m32 -fno-stack-protector -fno-pic -I. -c include/VM86/idt.c -o idt.o
gcc -m32 -fno-stack-protector -fno-pic -I. -c include/VM86/vm86.c -o vm86.o

gcc -m32 -fno-stack-protector -fno-pic -I. -c include/acpi/acpi.c -o acpi.o
gcc -m32 -fno-stack-protector -fno-pic -I. -c include/acpi/energy.c -o energy.o
gcc -m32 -fno-stack-protector -fno-pic -I. -c include/acpi/string.c -o string.o
gcc -m32 -fno-stack-protector -fno-pic -I. -c include/system.c -o system.o
gcc -m32 -fno-stack-protector -fno-pic -I. -c include/kb.c -o kb.o
gcc -m32 -fno-stack-protector -fno-pic -I. -c include/screen.c -o screen.o

# link
ld -m elf_i386 -T link.ld -o bootloader.bin bootloader.o kc.o cpu.o idt.o isrs.o ints.o vm86.o vm86asm.o acpi.o energy.o string.o system.o kb.o screen.o

# build iso image
cp bootloader.bin Slidoor/boot/
grub-mkrescue -o Slidoor.iso Slidoor/

# run
qemu-system-x86_64 -cdrom Slidoor.iso -debugcon stdio
