#include "string.h"

//#include <stddef.h>

void *faster_memcpy(void * _dest, const void *_src, uint32 _n)
{
    uint32 *sp = (uint32*) _src;
    uint32 *dp = (uint32*) _dest;
	//	_n /= sizeof(uint32);
    for(; _n != 0; _n--) {
		*dp++ = *sp++;
	}
	return _dest;
}

void faster_memset(void *_s, uint32 _c, size_t _n)
{
    uint32 *temp = (uint32*) _s;
    for ( ; _n != 0; _n--)
    {
		    *temp++ = _c;
	  }
}

int memcmp(const void *_s1, const void *_s2, size_t _n)
{
	const unsigned char *us1 = (const unsigned char *)_s1;
	const unsigned char *us2 = (const unsigned char *)_s2;
	while(_n-- != 0) {
		if(*us1 != *us2){
			return (*us1 < *us2) ? -1 : +1;
		}
		us1++;
		us2++;
	}
	return 0;
}

void *memcpy(void * _dest, void *_src, size_t _n)
{
    char *sp = (char*) _src;
    char *dp = (char*) _dest;
    for(; _n != 0; _n--) {
		*dp++ = *sp++;
	}
	return _dest;
}

void* memmove(void* dstptr, const void* srcptr, size_t size)
{
	unsigned char* dst = (unsigned char*) dstptr;
	const unsigned char* src = (const unsigned char*) srcptr;
	if ( dst < src )
		for ( size_t i = 0; i < size; i++ )
			dst[i] = src[i];
	else
		for ( size_t i = size; i != 0; i-- )
			dst[i-1] = src[i-1];
	return dstptr;
}


void *memset(void *_s, int _c, size_t _n)
{
    char *temp = (char*) _s;
    for ( ; _n != 0; _n--) {
		*temp++ = _c;
	}
    return _s;
}

int memcmp_fast(const void *_s1, const void *_s2, size_t _n)
{
	const uint32 *us1 = (const uint32 *)_s1;
	const uint32 *us2 = (const uint32 *)_s2;
	_n /= sizeof(uint32);
  int i = 0;
	while(_n-- != 0) {
		if(*us1 != *us2){
			return i;//(*us1 < *us2) ? -1 : +1;
		}
    ++i;
		us1++;
		us2++;
	}
	return 0;
}

void *memcpy_fast(void * _dest, const void *_src, size_t _n)
{
    uint32 *sp = (uint32*) _src;
    uint32 *dp = (uint32*) _dest;
		_n /= sizeof(uint32);
    for(; _n != 0; _n--) {
		*dp++ = *sp++;
	}
	return _dest;
}

void* memmove_fast(void* dstptr, const void* srcptr, size_t size)
{
	uint32* dst = (uint32*) dstptr;
	const uint32* src = (const uint32*) srcptr;
	size /= sizeof(uint32);
	if ( dst < src )
		for ( size_t i = 0; i < size; i++ )
			dst[i] = src[i];
	else
		for ( size_t i = size; i != 0; i-- )
			dst[i-1] = src[i-1];
	return dstptr;
}


void memset_fast(void *_s, int _c, size_t _n)
{
    uint32 *temp = (uint32*) _s;
		_n /= sizeof(uint32);
    for ( ; _n != 0; _n--) {
		*temp++ = _c;
	}
}

size_t strlen(const char *_s)
{
   size_t n;
   for(n=0; _s[n] != '\0'; n++) {
	   ;
   }
   return n;
}

int strcmp(const char *_s1, const char *_s2)
{
    for(; *_s1 == *_s2; ++_s1, ++_s2) {
        if(*_s1 == 0) {
            return 0;
		}
	}
    return *(unsigned char *)_s1 < *(unsigned char *)_s2 ? -1 : 1;
}

int strcmpx(const char *_s1, const char *_s2)
{
    for(; *_s1 == *_s2 || *_s1 - 32 == *_s2 || *_s1 == *_s2 - 32; ++_s1, ++_s2) {
        if(*_s1 == 0) {
            return 0;
		}
	}
    return *(unsigned char *)_s1 < *(unsigned char *)_s2 ? -1 : 1;
}

int strncmp(const char *_s1, const char *_s2, size_t _n)
{
	unsigned char uc1, uc2;
	if (_n==0)
		return 0;
	while (_n-- > 0 && *_s1 == *_s2) {
		if (_n==0 || *_s1 == '\0' ) {
			return 0;
		}
		_s1++;
		_s2++;
	}
	uc1 = (*(unsigned char *) _s1);
	uc2 = (*(unsigned char *) _s2);
	return ((uc1 < uc2) ? -1 : (uc1 > uc2));
}

char *strcpy(char *_dest, const char *_src)
{
    do {
      *_dest++ = *_src++;
    } while (*_src != 0);
  *_dest = '\0';
	return _dest;
}

char *strncpy(char *_dest, const char *_src, int size)
{
    do {
      *_dest++ = *_src++;
    } while (--size);/*/
  for(int i = 0; i < size; i++)
  {
    *_dest++ = *_src++;
  }//*/
  *_dest = '\0';
	return _dest;
}

char *strcat(char *_dest, const char *_src)
{
   size_t n;

   n = strlen(_dest);
   strcpy(_dest + n, _src);

   return _dest;
}

int strcmpy(const char *c1, const char *c2)
{
    uint32 buff=0;
    for(uint32 i=0;i<=strlen(c1);i++)
    {
        for(uint32 j=0;j<strlen(c2);j++)
        {
            if(c1[i+j]==c2[j])
            {
                buff+=1;
            }
        }
        if(buff==strlen(c2))
            return 0;
        else buff=0;
    }
    return 1;
}

char *strchr(const char *_s, int _c)
{
	while (*_s != (char)_c) {
		if (!*_s++) {
			return 0;
		}
	}
	return (char *)_s;
}

size_t strspn(const char *_s, const char *_accept)
{
	size_t ret=0;
	while(*_s && strchr(_accept,*_s++)) {
		ret++;
	}
	return ret;
}

size_t strcspn(const char *_s, const char *_reject)
{
	size_t ret=0;
	while(*_s) {
		if(strchr(_reject,*_s)) {
			return ret;
		} else {
			_s++,ret++;
		}
	}
	return ret;
}

char *strtok(char *_s, const char *_delim)
{
	static char* p=0;
	if(_s) {
		p=_s;
	} else if(!p) {
		return 0;
	}
	_s = p   + strspn(p,_delim);
	p  = _s  + strcspn(_s,_delim);
	if(p==_s) {
		return p=0;
	}
	p = *p ? *p=0,p+1 : 0;
	return _s;
}

char *strstr(const char *_haystack, const char *_needle)
{
	size_t needleLen;
	if(*_needle == '\0') {
		return (char *) _haystack;
	}
	needleLen = strlen(_needle);

	for(;(_haystack = strchr(_haystack, *_needle))!= NULL;_haystack++) {
		if (strncmp(_haystack, _needle, needleLen) == 0) {
			return (char *) _haystack;
		}
	}
	return NULL;
}

uint32 stroccr(char* str, const char c)
{
	uint32 i = 0;
	for(;*str!='\0';)
	{
		if(*str == c)	i++;
		++str;
	}
	return i;
}

uint16 strlength(string ch)
{
        uint16 i = 1;
        while(ch[i++]);  
        return --i;
}

/* Added in episode 3*/ /*This function compares two strings and returns true (1) if they are equal or false (0) if they are not equal */

uint8 strEql(string ch1,string ch2)                     
{
        uint8 result = 1;
        uint8 size = strlength(ch1);
        if(size != strlength(ch2)) result =0;
        else 
        {
        uint8 i = 0;
        for(i;i<=size;i++)
        {
                if(ch1[i] != ch2[i]) result = 0;
        }
        }
        return result;
}

